#-*- coding: utf-8 -*-
import time
import locale
from itertools import chain
import re

DEFAULT_LOCALE = locale.setlocale(locale.LC_ALL, '')
LOCALES = set([DEFAULT_LOCALE, 'C'])

PUNCTUATION = re.compile(r'[,:]$')
UTC_DATE_FORMAT = '%d/%m/%Y %H:%M:%S +0000'
DATE_PATTERNS = {'day': ['%d', '%x', '%Y-%m-%d'],
                  'month': ['%b', '%B', '%m', '%x', '%Y-%m-%d'],
                  'year': ['%Y', '%x', '%Y-%m-%d'],
                  'hour': ['%X'],
                }

def filter_date(value):
    """
    Return True if value is parsable with common time strptime.
    :param value: value we want to test against common time strptime formats.
    :type value: str
    """
    value_els = [re.sub(PUNCTUATION, '', v) for v in value.split()]
    date_els = []
    for l in LOCALES:
        locale.setlocale(locale.LC_ALL, l)
        for pattern in set(chain.from_iterable(DATE_PATTERNS.values())):
            for value_el in value_els:
                try:
                    if time.strptime(value_el, pattern):
                        date_els.append((value_el, pattern))
                except:
                    continue
        if len(date_els) > 0:
            els, patterns = zip(*date_els)
            is_date = True in [p in patterns for p in DATE_PATTERNS['day']] and \
                      True in [p in patterns for p in DATE_PATTERNS['month']] and \
                      True in [p in patterns for p in DATE_PATTERNS['year']] and \
                      True in [p in patterns for p in DATE_PATTERNS['hour']]
            if is_date:
                date = time.mktime(time.strptime(' '.join(els), ' '.join(patterns)))
                value = time.strftime(UTC_DATE_FORMAT, time.gmtime(date))
                break
            else:
                date_els = []

    locale.setlocale(locale.LC_ALL, DEFAULT_LOCALE)
    return value

def repr_value(value):
    """
    Coerce value to stripped string for ConfigParser setting function.
    :param value: value to coerce to string
    :type value: arbitrary type
    """
    if isinstance(value, list):
        value = ', '.join([repr_value(v) for v in value])
    elif isinstance(value, int):
        value = str(value).strip()
    else:
        value = filter_date(value).strip()
    return value

def eole23_version(eole_common_version):
    """
    Return EOLE version depending on eole-common package version
    :param eole_common_version: eole-common package version as returned by dpkg -l command
    :type eole_common_version: str starting with '2.3-eole'
    """
    mapping = {(139,):'2.3.1', 
               (142,):'2.3.2', 
               (143,):'2.3.3', 
               (145,):'2.3.4', 
               (146, 147):'2.3.5', 
               (148,):'2.3.6', 
               (150, 151):'2.3.7', 
               (152, 153):'2.3.8', 
               (154,):'2.3.9', 
               (155,):'2.3.10',
               (156, 157):'2.3.11',
               (158,):'2.3.12',
               (159,):'2.3.13',
               (160,):'2.3.14',
               (161, 162):'2.3.15',
               (163,):'2.3.16'}
    version =  [mapping[m] for m in mapping if int(eole_common_version.replace('2.3-eole','')) in m]
    if len(version) != 0:
        version = version[0]
    else:
        version = ''
    return version
